""" Kick off the cluster jobs to make monitoring maps for custom periods in
month."""

__author__ = "Carl Schreck"
__email__ = "carl_schreck@ncsu.edu"
__copyright__ = "Copyright 2020, North Carolina State University"
__license__ = "BSD-3.0"

import numpy as np
import os
import argparse
import subprocess
from datetime import datetime
import glob
import calendar
import cjs.fileio
import logging
from download_nclimgrid import download_nclimgrid

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=(datetime.today().year))
parser.add_argument('--month', default=(datetime.today().month - 1))
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
if month < 1:
    month += 12
    year -= 1

first_days = np.array([1, 10, 23, ])

days_in_month = calendar.monthrange(year, month)[1]
clobber = True

script_types = {
    'tavg': ['anom', 'rank'],
    'prcp': ['pct', 'rank'],
    'z500': ['mean_anom']
}

last_days = first_days[1:] - 1
last_days = np.append(last_days, days_in_month)
first_days = np.append(first_days, 1)
last_days = np.append(last_days, days_in_month)

logger.info('Here we go!')
fig_dir = f'/glfs0/anon-ftp/pub/carl/monitor/synoptic'
os.makedirs(fig_dir, exist_ok=True)
old_files = glob.glob(f'{fig_dir}/*')
for file in old_files:
    os.remove(file)
py_dir = os.getcwd()
log_dir = f'{py_dir}/log'
os.makedirs(log_dir, exist_ok=True)
log_files = glob.glob(f'{log_dir}/*')
for file in log_files:
    os.remove(file)

for var_name in script_types.keys():

    logger.info(f'Downloading {var_name}')
    if var_name == 'z500':
        file_name = f'hgt.{year}.nc'
        in_path = f'{os.environ["DATA_DIR"]}/ncep1/esrl/daily/{file_name}'
        ftp_path = (
            f'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis.dailyavgs/'
            f'pressure/{file_name}')
        download_err = cjs.fileio.download_file(ftp_path, in_path,
                                                clobber=clobber)
    else:
        in_path = download_nclimgrid(year, month, clobber=clobber)
        if 'prelim' in in_path:
            file_type = 'prelim'
        else:
            file_type = 'scaled'

    for first_day, last_day in zip(first_days, last_days):

        print(var_name)
        for script_type in script_types[var_name]:
            print(script_type)

            first_date = f'{year}-{month:02}-{first_day:02}'
            last_date = f'{year}-{month:02}-{last_day:02}'
            py_script = f'{var_name}_{script_type}.py'

            if var_name == 'z500':
                regions = ['namer', 'nhem']
                dpi = 387
                if first_day == 1 and last_day == days_in_month:
                    regions.append('global')
            else:
                regions = [None]
                dpi = 150

            for region in regions:
                job_name = f'{var_name}_{script_type}_{first_date}_{last_date}'
                py_option = (f'--first_date {first_date} '
                             f'--last_date {last_date} '
                             f'--dpi {dpi} --fig_dir {fig_dir}')
                if var_name != 'z500':
                    py_option = f'{py_option} --file_type {file_type}'

                if region is not None:
                    job_name += f'_{region}'
                    py_option += f' --region {region}'
                print(job_name)
                log_file = f'{log_dir}/{job_name}.log'
                err_file = f'{log_dir}/{job_name}.err'
                command = (f'bsub -J {job_name} -o {log_file} -e {err_file} '
                           f'-q allnodes -sp 75 {py_dir}/run_py.sh '
                           f'{py_dir} {py_script} "{py_option}"')
                subprocess.run(command, shell=True)

py_script = 'telecon_timeseries'
job_name = f'{py_script}'
print(job_name)
log_file = f'{log_dir}/{job_name}.log'
err_file = f'{log_dir}/{job_name}.err'
py_option = (
    f' --fig_dir {fig_dir} '
    f' --first_days {np.array2string(first_days[:-1])[1:-1]} '
)
command = (
    f'bsub -J {job_name} -o {log_file} -e {err_file} '
    f'-q allnodes -sp 75 {py_dir}/run_py.sh '
    f'{py_dir} {py_script} "{py_option}"'
)
subprocess.run(command, shell=True)

py_script = 'rmm/rmm_main'
job_name = f'rmm'
print(job_name)
log_file = f'{log_dir}/{job_name}.log'
err_file = f'{log_dir}/{job_name}.err'
py_option = (f'--first_date {year}-{month}-01 '
             f'--last_date {year}-{month}-{days_in_month} '
             f'--dpi {dpi} --fig_dir {fig_dir}')
command = (
    f'bsub -J {job_name} -o {log_file} -e {err_file} '
    f'-q allnodes -sp 75 {py_dir}/run_py.sh '
    f'{py_dir} {py_script} "{py_option}"'
)
subprocess.run(command, shell=True)


logger.info('Thank you, come again.')

