""" Kick off the cluster jobs to make monitoring maps for each day of the
month."""

__author__ = "Carl Schreck"
__email__ = "carl_schreck@ncsu.edu"
__copyright__ = "Copyright 2020, North Carolina State University"
__license__ = "BSD-3.0"

import numpy as np
import os
import argparse
import subprocess
from datetime import datetime
import glob
import calendar
import cjs.text
import cjs.fileio
from download_nclimgrid import download_nclimgrid

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=(datetime.today().year))
parser.add_argument('--month', default=(datetime.today().month - 1))
# parser.add_argument('--month', default=6)
parser.add_argument('--dpi', default=100)
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
dpi = int(args.dpi)
if month < 1:
    month += 12
    year -= 1

start_day = 1
days_in_month = calendar.monthrange(year, month)[1]
clobber = True

delta_days = [1, 7, 14, days_in_month]

script_types = {
    'tavg': 'anom',
    'prcp': 'pct',
    'z500': 'mean_anom'
}


cjs.text.tstamp('Here we go!')
for var_name in script_types.keys():

    cjs.text.tstamp(f'Downloading {var_name}')
    if var_name == 'z500':
        file_name = f'hgt.{year}.nc'
        in_path = f'{os.environ["DATA_DIR"]}/ncep1/esrl/daily/{file_name}'
        ftp_path = (
            f'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis.dailyavgs/'
            f'pressure/{file_name}')
        download_err = cjs.fileio.download_file(ftp_path, in_path, clobber=True)
    else:
        in_path = download_nclimgrid(year, month, clobber=clobber)
        if 'prelim' in in_path:
            file_type = 'prelim'
        else:
            file_type = 'scaled'

    for delta_day in delta_days:
        fig_dir = f'synoptic/{var_name}-{delta_day}'
        os.makedirs(fig_dir, exist_ok=True)
        old_files = glob.glob(f'{fig_dir}/*')
        for file in old_files:
            os.remove(file)

        first_days = np.arange(start_day, days_in_month - delta_day + 2, 1)
        last_days = np.arange(start_day + delta_day - 1, days_in_month + 1, 1)
        for first_day, last_day in zip(first_days, last_days):

            first_date = f'{year}-{month:02}-{first_day:02}'
            last_date = f'{year}-{month:02}-{last_day:02}'

            py_dir = os.getcwd()
            py_script = f'{var_name}_{script_types[var_name]}.py'
            log_dir = f'{py_dir}/log'
            os.makedirs(log_dir, exist_ok=True)
            job_name = f'{var_name}_{first_date}_{last_date}'
            log_file = f'{log_dir}/{job_name}.log'
            err_file = f'{log_dir}/{job_name}.err'
            try:
                os.remove(log_file)
            except OSError:
                pass
            try:
                os.remove(err_file)
            except OSError:
                pass

            py_option = f'--first_date {first_date} --last_date {last_date} ' \
                        f'--dpi {dpi} --fig_dir {fig_dir}'
            if var_name != 'z500':
                py_option = f'{py_option} --file_type {file_type}'

            command = f'bsub -J {job_name} -o {log_file} -e {err_file} ' \
                      f'-q allnodes -sp 75 {py_dir}/run_py.sh ' \
                      f'{py_dir} {py_script} "{py_option}"'
            subprocess.Popen(command, shell=True)




cjs.text.tstamp('Thank you, come again.')

