""" Calculate the statistics for the monthly TC report. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import cjs.text
from cjs import ibtracs
from datetime import datetime
import numpy as np
import pandas as pd
import calendar
import json
import os

# Set up logging
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

cjs.text.tstamp('Here we go!')
# These are some parameters that could be useful to have up top
# basins = {'NA': 'North Atlantic', 'EP': 'East Pacific'}
basins = {'NA': 'North Atlantic', 'EP': 'East Pacific', 'WP': 'West Pacific',
          'NI': 'North Indian', 'WMOSI': 'South Indian',
          'WMOAUS': 'Australia', 'WMOSP': 'Southwest Pacific', 'GL': 'Global'}
# basins = {'WMOAUS': 'Australia', }
start_years = {'NA': 1851, 'EP': 1945, 'WP': 1945, 'NI': 1972, 'WMOSI': 1956,
               'WMOAUS': 1956, 'WMOSP': 1956, 'GL': 1972}
end_year = datetime.today().year-1
# end_year = 2015

out_dir = f'/glfs0/anon-ftp/pub/carl/monitor/tropical-cyclones-{end_year}13'
os.makedirs(out_dir, exist_ok=True)

out_dict = {}
symbol_dict = {}
note_symbols = ['<sup>&dagger;</sup>', '<sup>&Dagger;</sup>', '<sup>&Vert;</sup>',
                '<sup>**</sup>', '<sup>&dagger;&dagger;</sup>',
                '<sup>&Dagger;&Dagger;</sup>', '<sup>&Vert;&Vert;</sup>',
                ]
note_counter = 0

cjs.text.tstamp('Downloading/opening data')
ibt = ibtracs.Ibtracs('ALL', download_data=False)
# ibt = ibtracs.Ibtracs('since1980', download_data=False)
# ibt = ibtracs.Ibtracs('last3years', download_data=True)
# ibt_path = (f'{os.getenv("DATA_DIR")}/ibtracs/v04r01/IBTrACS.'
#             f'ALL.v04r01.nc')
# ibt = ibtracs.Ibtracs(ibt_path, download_data=False)

for basin, basin_name in basins.items():
    out_dict[basin_name] = {
        'storms': {
            'list': [],
            'notes': {},
        },
        'counts': {
            'season': {
                'stats': {},
                'notes': {},
            },
        }
    }

    cjs.text.tstamp(f'Individual {ibt.basin_names[basin]} Storms:')
    if basin == 'GL':
        is_in_year = ibt.ds['time.year'] == end_year
    else:
        is_in_year = ibt.ds['season'] == end_year
    is_tropical = ibt.find_tropical(35)
    is_in_basin = ibt.check_basin(basin)
    is_eligible = is_in_year & is_tropical & is_in_basin
    any_eligible = np.any(is_eligible, axis=1)
    for storm in ibt.ds.storm[any_eligible]:
        storm_dict = {}
        print(f'name: {ibt.ds.name[storm].item().decode().title()}')
        storm_dict['name'] = {
            'value': ibt.ds.name[storm].item().decode().title()}
        sid = ibt.ds.sid[storm].item().decode().title()
        print(f'id: {sid}')
        storm_dict['url'] = {
            'value': f'https://ncics.org/ibtracs/index.php?name=v04r01-'
                     f'{sid}'}

        ibt.savefig_one_storm(out_dir, target_sid=sid)

        max_wind = ibt.ds.usa_wind[storm, is_tropical[storm, :]].max().item()
        min_pres = ibt.ds.usa_pres[storm, is_tropical[storm, :]].min().item()
        is_max_wind = (max_wind == ibt.ds.usa_wind[storm, :])
        is_min_pres = (min_pres == ibt.ds.usa_pres[storm, :])
        # use time of peak pressure to decide nature
        nature = ibt.ds.nature[storm, is_min_pres][0].values
        print(f'category: {ibt.ss_cat(max_wind, nature)}')
        storm_dict['maxCategory'] = {'value': ibt.ss_cat(max_wind, nature,
                                                         long_not_short=False)}

        max_wind_mph = round(max_wind * 1.15078 / 5) * 5
        print(f'max wind: {max_wind} kt / {max_wind_mph} mph')
        storm_dict['maxWinds'] = {'value': max_wind_mph}

        print(f'min pressure: {min_pres}')
        storm_dict['minPressure'] = {'value': min_pres}

        # check basin of max wind
        if not any(is_in_basin[storm, is_max_wind]):
            max_wind_basin = \
                ibt.ds['basin'][storm, is_max_wind][0].item().decode()
            note_text = f'Value occurred in the' \
                        f' {ibt.basin_names[max_wind_basin]}'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['maxWinds']:
                storm_dict['maxWinds']['note'] += symbol_dict[note_text]
            else:
                storm_dict['maxWinds']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][
                note_symbols[note_counter]] = \
                f'Value occurred in the {ibt.basin_names[max_wind_basin]}'
            note_counter += 1

        # check basin of min pressure
        if not any(is_in_basin[storm, is_min_pres]):
            min_pres_basin = \
                ibt.ds['basin'][storm, is_min_pres][0].item().decode()
            note_text = f'Value occurred in the' \
                        f' {ibt.basin_names[min_pres_basin]}'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['minPressure']:
                storm_dict['minPressure']['note'] += symbol_dict[note_text]
            else:
                storm_dict['minPressure']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        min_time_ind = np.where(is_tropical[storm, :])[0].min()
        max_time_ind = np.where(is_tropical[storm, :])[0].max()
        min_time_str = pd.to_datetime(
            ibt.ds.time[storm, min_time_ind].item()).strftime('%Y%m%d')
        max_time_str = pd.to_datetime(
            ibt.ds.time[storm, max_time_ind].item()).strftime('%Y%m%d')
        storm_dict['begDate'] = {'value': min_time_str}
        storm_dict['endDate'] = {'value': max_time_str}
        print(f'{min_time_str} - {max_time_str}')
        print()
        out_dict[basin_name]['storms']['list'].append(storm_dict)

    out_dict[basin_name]['storms']['list'] = sorted(
        out_dict[basin_name]['storms']['list'],
        key=lambda x: x['begDate']['value']
    )
    print()
    print()

    period = 'season'
    months = range(1, 13)
    print(f'Months: {months}')

    start_year = start_years[basin]
    print('\n')
    cjs.text.tstamp(f'{ibt.basin_names[basin]}: {calendar.month_name[months[0]]}'
               f' - {calendar.month_name[months[-1]]} : {start_year}')
    cjs.text.tstamp('Calculating ACE')
    ace = ibt.calc_ace_yyyymm(target_basin=basin,
                              start_year=start_year, end_year=end_year,
                              months=months).sum(dim='month')
    ace.name = 'ACE'
    cjs.text.tstamp('Count 35')
    count_35 = ibt.count_yyyymm(thresh=35, target_basin=basin,
                                start_year=start_year, end_year=end_year,
                                months=months, count_passing=True
                                )
    count_35.name = 'ge35kts'

    cjs.text.tstamp('Count 64')
    count_64 = ibt.count_yyyymm(thresh=64, target_basin=basin,
                                start_year=start_year, end_year=end_year,
                                months = months, count_passing = True
                                )
    count_64.name = 'ge64kts'
    cjs.text.tstamp('Count 96')
    count_96 = ibt.count_yyyymm(thresh=96, target_basin=basin,
                                start_year=start_year, end_year=end_year,
                                months=months, count_passing=True
                                )
    count_96.name = 'ge96kts'

    for activity in [count_35, count_64, count_96, ace]:
        current = activity.loc[end_year].item()
        rank = sum(current < activity.loc[1981:].values) + 1
        tied = sum(current == activity.loc[1981:].values) - 1
        climo = activity.loc[1991:2020].mean().item()
        record = activity.loc[:end_year].max().item()
        record_years = activity.year[activity == record].values.tolist()
        print()
        print(f'{ibt.basin_names[basin]}: {activity.name}')
        print(f'{end_year}: {current}')
        stats = {}
        stats['count'] = {'value': current}
        print(f'Rank: {rank}')
        stats['rank'] = {'value': int(rank)}
        print(f'Tied with: {tied}')
        if tied > 0:
            stats['rank']['tie'] = int(tied)
        print(f'Average: {climo}')
        stats['mean'] = {'value': climo}
        print(f'Record: {record}')
        stats['record'] = {'value': record}
        if record > 0:
            # print(f'Years: {np.array2string(record_years, separator=", ")}')
            if len(record_years) > 4:
                stats['record']['years'] = [
                    f'{len(record_years)} {period}s']
            else:
                print(f'Years: '
                      f'{", ".join([str(year) for year in record_years])}')
                stats['record']['years'] = record_years
        if record_years[0] == end_year:
            # New record!
            old_record = activity.loc[:end_year-1].max().item()
            old_record_years = activity.year[
                activity == old_record].values.tolist()
            stats['record']['note'] = note_symbols[note_counter]
            if len(old_record_years) > 4:
                note_text = f'Previous record {old_record} ' \
                            f'({len(old_record_years)} {period}s)'
            else:
                old_record_years_string = ", ".join([
                    str(year) for year in old_record_years])
                note_text = f'Previous record {old_record} ' \
                            f'({old_record_years_string})'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            stats['record']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['counts'][period]['notes'][
                symbol_dict[note_text]] = note_text
        out_dict[basin_name]['counts'][period]['stats'][
            activity.name] = stats
        print(out_dict)


cjs.text.tstamp('Writing tropics')
out_path = f'{out_dir}/tropical-cyclones-{end_year}13.json'
with open(out_path, 'w') as out_file:
    json.dump(out_dict, out_file, indent=2)

cjs.text.tstamp('Thank you, come again.')

