""" Draw maps of TC tracks for the latest month and for the season to date."""

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import cjs.text
import cjs.graphics
from cjs import ibtracs
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
from datetime import datetime
import calendar
import os
import argparse
import logging

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

parser = argparse.ArgumentParser()
parser.add_argument('--year', default=datetime.today().year)
parser.add_argument('--month', default=datetime.today().month - 1)
parser.add_argument('--out_dir', default='tropics/')
# parser.add_argument('--year', default=2021)
# parser.add_argument('--month', default=12)
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
out_dir = args.out_dir
# month = 1
# year = 2022
end_month = month + 1
end_year = year
if month < 1:
    month += 12
    year -= 1

os.makedirs(out_dir, exist_ok=True)

extents = {
    'NA': [360 - 105, 360 - 10, 0, 60],
    'EP': [175, 360 - 80, 0, 35],
    'WP': [95, 185, 0, 60],
    'NI': [40, 105, 0, 35],
    'WMOSI': [20, 95, -45, 0],
    'WMOAUS': [85, 165, -45, 0],
    'WMOSP': [155, 360 - 120, -60, 0],
    'GL': [1, 359, -60, 60],
}

cjs.text.tstamp('Here we go!')

# ibt = ibtracs.Ibtracs('last3years', download_data=True)
ibt_path = (f'{os.getenv("DATA_DIR")}/ibtracs/v04r01/IBTrACS.'
            f'last3years.v04r01.nc')
ibt = ibtracs.Ibtracs(ibt_path, download_data=False)

# for basin in ['WMOAUS']:
for basin in extents.keys():
    for period in ['month', 'season']:
        cjs.text.tstamp(f'{basin}: {period}')
        fig, ax = plt.subplots(nrows=1, ncols=1, dpi=300,
                                subplot_kw={
                                    'projection': ccrs.PlateCarree(
                                        central_longitude=180),
                                    # 'anchor': 'W',
                                })
        # ax = plt.axes(projection=ccrs.PlateCarree(central_longitude=240))
        if basin != 'GL':
            cjs.graphics.add_map(ax=ax, edgecolor='0.5', zorder=0)
        else:
            cjs.graphics.add_map(ax=ax, edgecolor='0.5', zorder=0,
                                 scale='110m', countries_on=False,
                                 states_on=False)

        if period == 'month':
            start_month = month
            start_year = year
            ax.set_title(f'{ibt.basin_names[basin]}\n'
                         f'{calendar.month_name[month]} {year}')
        else:
            if basin in ibt.sh_basins:
                start_month = 7
                if month < 7:
                    start_year = year - 1
                else:
                    start_year = year
            else:
                start_month = 1
                start_year = year
            if start_year == year:
                ax.set_title(f'{ibt.basin_names[basin]}\n'
                             f'{calendar.month_abbr[start_month]}'
                             f'\u2013{calendar.month_abbr[month]} {year}')
            else:
                ax.set_title(f'{ibt.basin_names[basin]}\n'
                             f'{calendar.month_abbr[start_month]} {start_year}'
                             f' \u2013 '
                             f'{calendar.month_abbr[month]} {year}')

        add_date_labels = (period=='month') and (basin != 'GL')
        add_letter_labels = basin != 'GL'
        if (basin == 'GL') or (period == 'season'):
            markercolors = 'none'
        else:
            markercolors = 'wind'
        atcf_legend, storm_extent = ibt.map_storms(
            ax=ax,
            start_time=f'{start_year}-'
                       f'{start_month}-01',
            end_time=f'{end_year}-{end_month}-01',
            target_basin=basin, linecolor='wind',
            markercolor=markercolors,
            add_date_labels=add_date_labels,
            add_letter_labels=add_letter_labels,
            atcf_not_letter=True,
            add_legend=False)

        cjs.graphics.expand_extent(extents[basin], storm_extent, ax=ax)

        if basin == 'GL':
            lat_interval = 30
            lon_interval = 45
        else:
            lat_interval = 20
            lon_interval = 20
        cjs.graphics.add_map_ticks(ax=ax, lon_interval=lon_interval,
                                   lat_interval=lat_interval)


        plt.subplots_adjust(left=0.25, right=0.75)
        ax_pos = ax.get_position()
        # lg_ax = fig.add_axes([ax_pos.x1 + 0.05, ax_pos.y0, 0.0,
        #                       ax_pos.y1-ax_pos.y0])
        # lg_ax.axis('off')
        ibt.add_track_legend(ax=fig, loc='upper left',
                             bbox_to_anchor=(ax_pos.x1+0.05, ax_pos.y1),
                             markercolor='none')

        if basin != 'GL':
            fig.text(ax_pos.x0-0.10, ax_pos.y1, atcf_legend[:-1], fontsize=6,
                     va='top', ha='right', ma='left',
                     bbox=dict(facecolor='white', edgecolor='dimgray',
                               linewidth=0.5,
                               boxstyle='square,pad=0.5'))

        plt.savefig(f'{out_dir}/{basin.replace("WMO", "")}'
                    f'_{period}_{year}{month:02}.png',
                    dpi=300,
                    bbox_inches='tight')
        # plt.show()
        plt.close()


cjs.text.tstamp('Thank you, come again.')
