""" Calculate the statistics for the monthly TC report. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import cjs.text
from cjs import ibtracs
from datetime import datetime
import argparse
import numpy as np
import pandas as pd
import calendar
import json
import os

def get_storm_basins(ibt, storm, basins, omit_basins='GL', date_times=None):
    passed_basins = []

    # Loop over all basins
    for basin_code, basin_name in basins.items():
        if basin_code in omit_basins:
            continue  # Skip the omitted basins

        # Get the basin data using ibt.check_basin for the current basin
        basin_data = ibt.check_basin(target_basin=basin_code)

        # If times are provided, select data for the specified times
        if date_times is not None:
            basin_data = basin_data.sel(date_time=date_times)

        # Check if the storm appears in this basin at any time
        if basin_data.sel(storm=storm).any():
            passed_basins.append(basin_name)

    # Convert the list of basin names to a comma-separated string
    return ', '.join(passed_basins) if passed_basins else "No basins found"


def get_unique_months_string(ibt, storm, end_month, date_times=None):
    # Extract all unique months the storm passes through
    # If times are provided, select data for the specified times
    all_months = ibt.ds['time.month'][storm]
    if date_times is not None:
        all_months = all_months.sel(date_time=date_times)
    unique_months = set(all_months.values)  # Convert to a set to ensure uniqueness

    # Exclude the end_month
    unique_months.discard(end_month)

    # Sort the months and convert to month names
    unique_month_names = [calendar.month_name[int(month)] for month in sorted(unique_months)]

    # Join the month names into a comma-separated string
    return ', '.join(unique_month_names)


# Set up logging
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

base_symbols = ['&dagger;', '&Dagger;', '&Vert;', '&sect;', '&para;', '*']
note_symbols = ['<sup>' + sym * n + '</sup>'
                for n in range(1, 5)
                for sym in base_symbols
                if not (sym == '*' and n == 1)]

cjs.text.tstamp('Here we go!')
# These are some parameters that could be useful to have up top
basins = {'NA': 'North Atlantic', 'EP': 'East Pacific', 'WP': 'West Pacific',
          'NI': 'North Indian', 'WMOSI': 'South Indian',
          'WMOAUS': 'Australia', 'WMOSP': 'Southwest Pacific', 'GL': 'Global'}
# basins = {'WMOSI': 'South Indian',
#           'WMOAUS': 'Australia', 'WMOSP': 'Southwest Pacific', 'GL': 'Global'}
# basins = {'NA': 'North Atlantic', 'EP': 'East Pacific'}
# basins = {'NA': 'North Atlantic', }
# basins = {'EP': 'East Pacific', }
# basins = {'GL': 'Global', }
# basins = {'WMOSP': 'Southwest Pacific', }
start_years = {'NA': 1851, 'EP': 1945, 'WP': 1945, 'NI': 1972, 'WMOSI': 1956,
               'WMOAUS': 1956, 'WMOSP': 1956, 'GL': 1972}
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=datetime.today().year)
parser.add_argument('--month', default=datetime.today().month - 1)
parser.add_argument('--out_dir', default='tropics/')
# parser.add_argument('--year', default=2021)
# parser.add_argument('--month', default=12)
args = parser.parse_args()
curr_year = int(args.year)
end_month = int(args.month)
out_dir = args.out_dir
# end_month = 0
if end_month < 1:
    end_month += 12
    curr_year -= 1

os.makedirs(out_dir, exist_ok=True)

out_dict = {}
symbol_dict = {}

cjs.text.tstamp('Downloading/opening data')
# curr_year_source = 'tcvitals'
# curr_year_source = 'last3years'
curr_year_source = (f'{os.getenv("DATA_DIR")}/ibtracs/v04r01/IBTrACS.'
                    f'last3years.v04r01.nc')
# ibt = ibtracs.Ibtracs('ALL', download_data=False)
tcv = ibtracs.Ibtracs(curr_year_source, download_data=False)
# ibt = ibtracs.Ibtracs('since1980', download_data=False)
# ibt = ibtracs.Ibtracs('last3years', download_data=True)
ibt_path = (f'{os.getenv("DATA_DIR")}/ibtracs/v04r01/IBTrACS.'
            f'ALL.v04r01.nc')
ibt = ibtracs.Ibtracs(ibt_path, download_data=False)
# tcv = ibtracs.Ibtracs(ibt_path, download_data=False)

for basin, basin_name in basins.items():
    # for basin, basin_name in {'EP': basins['EP']}.items():
    out_dict[basin_name] = {
        'storms': {
            'list': [],
            'notes': {},
        },
        'counts': {
            'month': {
                'stats': {},
                'notes': {},
            },
            'season': {
                'stats': {},
                'notes': {},
            },
        }

    }

    cjs.text.tstamp(f'Individual {ibt.basin_names[basin]} Storms:')
    is_in_months = ((ibt.ds['time.month'] == end_month)
                    & (ibt.ds['time.year'] == curr_year))
    is_tropical = ibt.find_tropical(35)
    is_in_basin = ibt.check_basin(basin)
    is_main = ibt.is_main()
    is_eligible = is_in_months & is_tropical & is_in_basin & is_main
    any_eligible = np.any(is_eligible, axis=1)
    for storm in ibt.ds.storm[any_eligible]:
        storm_dict = {}
        print(f'name: {ibt.ds.name[storm].item().decode().title()}')
        storm_dict['name'] = {
            'value': ibt.ds.name[storm].item().decode().title()}
        sid = ibt.ds.sid[storm].item().decode().title()
        print(f'id: {sid}')
        storm_dict['url'] = {
            'value': f'https://ncics.org/ibtracs/index.php?name=v04r01-'
                     f'{sid}'
            # 'value': f'/monitoring-content/sotc/tropical-cyclones/'
            #          f'{curr_year}/{end_month:02}/{sid}.png'
        }

        if basin != 'GL':
            ibt.savefig_one_storm(out_dir, target_sid=sid)

        max_wind = ibt.ds.usa_wind[storm, is_tropical[storm, :]].max().item()
        min_pres = ibt.ds.usa_pres[storm, is_tropical[storm, :]].min().item()
        is_max_wind = (max_wind == ibt.ds.usa_wind[storm, :])
        is_min_pres = (min_pres == ibt.ds.usa_pres[storm, :])
        # use time of peak pressure to decide nature
        nature = ibt.ds.nature[storm, is_min_pres][0].values
        print(f'category: {ibt.ss_cat(max_wind, nature)}')
        storm_dict['maxCategory'] = {'value': ibt.ss_cat(max_wind, nature,
                                                         long_not_short=False)}

        max_wind_mph = round(max_wind * 1.15078 / 5) * 5
        print(f'max wind: {max_wind} kt / {max_wind_mph} mph')
        storm_dict['maxWinds'] = {'value': max_wind_mph}

        print(f'min pressure: {min_pres}')
        storm_dict['minPressure'] = {'value': min_pres}

        ibt.calc_fix_ace()
        storm_ace = ibt.ds1d.ace[storm == ibt.ds1d.storm]
        summed_ace = storm_ace.sum().item()
        storm_dict['stormACE'] = {'value': f'{summed_ace:.1f}'}
        print(f'storm ACE: {summed_ace:.1f}')

        # check month of max wind
        if not is_in_months[storm, is_max_wind].any():
            max_wind_month = int(
                ibt.ds['time.month'][storm, is_max_wind][0].item())
            note_text = f'Value occurred in ' \
                        f'{calendar.month_name[max_wind_month]}.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            storm_dict['maxWinds']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        # check basin of max wind
        if not is_in_basin[storm, is_max_wind].any():
            max_wind_basin = get_storm_basins(ibt, storm, basins,
                                              omit_basins=['GL', basin],
                                              date_times=is_max_wind)
            note_text = f'Value occurred in the {max_wind_basin} basin.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['maxWinds']:
                storm_dict['maxWinds']['note'] += symbol_dict[note_text]
            else:
                storm_dict['maxWinds']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        # check month of min pressure
        if not is_in_months[storm, is_min_pres].any():
            min_pres_month = int(
                ibt.ds['time.month'][storm, is_min_pres][0].item())
            note_text = f'Value occurred in' \
                        f' {calendar.month_name[min_pres_month]}.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            storm_dict['minPressure']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        # check basin of min pressure
        if not is_in_basin[storm, is_min_pres].any():
            min_pres_basin = get_storm_basins(ibt, storm, basins,
                                              omit_basins=['GL', basin],
                                              date_times=is_min_pres)
            note_text = f'Value occurred in the {min_pres_basin} basin.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['minPressure']:
                storm_dict['minPressure']['note'] += symbol_dict[note_text]
            else:
                storm_dict['minPressure']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        # Check month of ACE
        ace_ind = np.where(storm_ace>0)[0]
        if not is_in_months[storm, ace_ind].all():
            other_month_string = get_unique_months_string(
                    ibt, storm, end_month, date_times=ace_ind)
            note_text = f'Some ACE occurred in {other_month_string}.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['stormACE']:
                storm_dict['stormACE']['note'] += symbol_dict[note_text]
            else:
                storm_dict['stormACE']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text

        # Check basin of ACE
        if not is_in_basin[storm, ace_ind].all():
            ace_basin = get_storm_basins(ibt, storm, basins,
                                         omit_basins=['GL', basin],
                                         date_times=ace_ind)
            note_text = f'Some ACE occurred in the {ace_basin} basin.'
            if note_text not in symbol_dict.keys():
                symbol_dict[note_text] = note_symbols[len(symbol_dict)]
            if 'note' in storm_dict['stormACE']:
                storm_dict['stormACE']['note'] += symbol_dict[note_text]
            else:
                storm_dict['stormACE']['note'] = symbol_dict[note_text]
            out_dict[basin_name]['storms']['notes'][symbol_dict[note_text]] = \
                note_text


        min_time_ind = np.where(is_tropical[storm, :])[0].min()
        max_time_ind = np.where(is_tropical[storm, :])[0].max()
        min_time_str = pd.to_datetime(
            ibt.ds.time[storm, min_time_ind].item()).strftime('%Y%m%d')
        max_time_str = pd.to_datetime(
            ibt.ds.time[storm, max_time_ind].item()).strftime('%Y%m%d')
        storm_dict['begDate'] = {'value': min_time_str}
        storm_dict['endDate'] = {'value': max_time_str}
        print(f'{min_time_str} - {max_time_str}')
        print()

        out_dict[basin_name]['storms']['list'].append(storm_dict)

    out_dict[basin_name]['storms']['list'] = sorted(
        out_dict[basin_name]['storms']['list'],
        key=lambda x: x['begDate']['value']
    )
    print()
    print()
    for period in ['month', 'season']:
        # for period in ['month']:
        print(period)
        if period == 'season':
            period_name = 'season'
            if basin in ['WMOSI', 'WMOAUS', 'WMOSP']:
                if end_month >= 7:
                    months = range(7, end_month + 1)
                else:
                    months = np.append(range(7, 13), range(1, end_month + 1))
            else:
                months = range(1, end_month + 1)
        elif period == 'month':
            months = [end_month]
            period_name = calendar.month_name[end_month]
        if basin in ['WMOSI', 'WMOAUS', 'WMOSP']:
            if end_month > 6:
                end_year = curr_year + 1
            else:
                end_year = curr_year
            year_var_name = 'season'
        else:
            end_year = curr_year
            year_var_name = 'time.year'
        print(f'Months: {months}')

        start_year = start_years[basin]
        print('\n')
        cjs.text.tstamp(
            f'{ibt.basin_names[basin]}: {calendar.month_name[months[0]]}'
            f' - {calendar.month_name[months[-1]]} : {start_year}')
        cjs.text.tstamp('Calculating ACE')
        ace = ibt.calc_ace_yyyymm(target_basin=basin,
                                  start_year=start_year, end_year=end_year,
                                  months=months).sum(dim='month')
        ace.loc[end_year] = tcv.calc_ace_yyyymm(target_basin=basin,
                                                start_year=end_year,
                                                end_year=end_year,
                                                months=months
                                                ).sum(dim='month').item()
        ace.name = 'ACE'
        cjs.text.tstamp('Count 35')
        count_35 = ibt.count_yyyymm(thresh=35, target_basin=basin,
                                    start_year=start_year, end_year=end_year,
                                    months=months, count_passing=True,
                                    year_var_name=year_var_name,
                                    )
        count_35.loc[end_year] = tcv.count_yyyymm(
            thresh=35, target_basin=basin,
            start_year=end_year, end_year=end_year,
            months=months, count_passing=True,
            year_var_name=year_var_name,
        ).item()
        count_35.name = 'ge35kts'

        cjs.text.tstamp('Count 64')
        count_64 = ibt.count_yyyymm(thresh=64, target_basin=basin,
                                    start_year=start_year, end_year=end_year,
                                    months=months, count_passing=True,
                                    year_var_name=year_var_name,
                                    )
        count_64.loc[end_year] = tcv.count_yyyymm(
            thresh=64, target_basin=basin,
            start_year=end_year, end_year=end_year,
            months=months, count_passing=True,
            year_var_name=year_var_name,
        ).item()
        count_64.name = 'ge64kts'
        cjs.text.tstamp('Count 96')
        count_96 = ibt.count_yyyymm(thresh=96, target_basin=basin,
                                    start_year=start_year, end_year=end_year,
                                    months=months, count_passing=True,
                                    year_var_name=year_var_name,
                                    )
        count_96.loc[end_year] = tcv.count_yyyymm(
            thresh=96, target_basin=basin,
            start_year=end_year, end_year=end_year,
            months=months, count_passing=True,
            year_var_name=year_var_name,
        ).item()
        count_96.name = 'ge96kts'

        for activity in [count_35, count_64, count_96, ace]:
            current = activity.loc[end_year].item()
            rank = sum(current < activity.loc[1981:].values) + 1
            tied = sum(current == activity.loc[1981:].values) - 1
            climo = activity.loc[1991:2020].mean().item()
            record = activity.loc[:end_year].max().item()
            record_years = activity.year[activity == record].values.tolist()
            print()
            print(f'{ibt.basin_names[basin]}: {activity.name}')
            print(f'{end_year}: {current}')
            stats = {}
            stats['count'] = {'value': current}
            print(f'Rank: {rank}')
            stats['rank'] = {'value': int(rank)}
            print(f'Tied with: {tied}')
            if tied > 0:
                stats['rank']['tie'] = int(tied)
            print(f'Average: {climo}')
            stats['mean'] = {'value': climo}
            print(f'Record: {record}')
            stats['record'] = {'value': record}
            if record > 0:
                # print(f'Years: {np.array2string(record_years, separator=", ")}')
                if len(record_years) > 4:
                    stats['record']['years'] = [
                        f'{len(record_years)} {period_name}s']
                else:
                    print(f'Years: '
                          f'{", ".join([str(year) for year in record_years])}')
                    stats['record']['years'] = record_years
            if record_years[0] == end_year:
                # New record!
                old_record = activity.loc[:end_year - 1].max().item()
                if type(old_record) == int:
                    old_record_string = str(old_record)
                else:
                    old_record_string = f'{old_record:.1f}'
                old_record_years = activity.year[
                    activity == old_record].values.tolist()
                if len(old_record_years) > 4:
                    note_text = f'Previous record {old_record_string} ' \
                                f'({len(old_record_years)} {period_name}s)'
                else:
                    old_record_years_string = ", ".join([
                        str(year) for year in old_record_years])
                    note_text = f'Previous record {old_record_string} ' \
                                f'({old_record_years_string})'
                if note_text not in symbol_dict.keys():
                    symbol_dict[note_text] = note_symbols[len(symbol_dict)]
                stats['record']['note'] = symbol_dict[note_text]
                out_dict[basin_name]['counts'][period]['notes'][
                    symbol_dict[note_text]] = note_text
            out_dict[basin_name]['counts'][period]['stats'][
                activity.name] = stats
            print(out_dict)

cjs.text.tstamp('Writing tropics')
out_path = f'{out_dir}/tropical-cyclones-{end_year}{end_month:02}.json'
# out_path = f'{out_dir}/tropical-cyclones-{end_year}{end_month:02}.' \
#            f'{curr_year_source}.json'
with open(out_path, 'w') as out_file:
    json.dump(out_dict, out_file, indent=2)

cjs.text.tstamp('Thank you, come again.')
