""" Draw the steps for one temperature variable. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2020, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from matplotlib import cm
from mpl_toolkits.axes_grid1 import make_axes_locatable
import os
import numpy as np
import cartopy.crs as ccrs
import argparse
from datetime import datetime
import calendar
import cjs.text
import cjs.graphics
from download_nclimgrid import download_nclimgrid
# from cjs import ibtracs

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=(datetime.today().year))
# parser.add_argument('--month', default=(datetime.today().month - 1))
parser.add_argument('--month', default=6)
parser.add_argument('--first_day', default=1)
parser.add_argument('--last_day', default=30)
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
if month < 1:
    month += 12
    year -= 1
first_day = int(args.first_day)
last_day = int(args.last_day)

time_range = slice(f'{year}-{month:02}-{first_day:02}',
                   f'{year}-{month:02}-{last_day:02}')

title = f'Hurricane Eta'

extent = cjs.graphics.conus_albers_extent
# extent = [-90.0, -70.0, 24.0, 38.0]
# extent = [-102.0, -90.0, 25.5, 35.0]
central_longitude = np.mean(extent[0:2])
# levels = [0.1, 0.5, 1, 5, 10, 25, 50, 75, 100, 150, 200]
mm_to_inches = 1 / 25.4
# mm_to_inches = 0.04
# levels = np.array([1, 5, 10, 25, 50, 75, 100, 150, ]) * mm_to_inches
levels = [0.001, 0.25, 0.5, 1, 2, 3, 4, 5, 6]
# levels = [2, 4, 6, 8, 10, 12]
# levels = np.arange(1,8)

cjs.text.tstamp('Here we go!')

cjs.text.tstamp('Downloading')
clobber = False
(in_path, file_type) = download_nclimgrid(year, month, clobber=clobber)


cmap = cm.get_cmap('GnBu')
newcolors= cmap(np.linspace(0, 1, 256))[28:]
newcolors[0] = [1, 1, 1, 1]
precip_cmap = mcolors.ListedColormap(newcolors)




cjs.text.tstamp('Read and average totals')
ds = xr.open_dataset(in_path)
data = ds['prcp'].sel(time=time_range).sum(dim='time')
data *= mm_to_inches

fig, ax = plt.subplots(1, 1, subplot_kw=dict(projection=ccrs.AlbersEqualArea(
    central_longitude=central_longitude, standard_parallels=extent[3:])))
cjs.graphics.add_conus_map(ax=ax, facecolor='none')
plot = data.plot.imshow(ax=ax, transform=ccrs.PlateCarree(),
                        add_colorbar=False,
                        cmap=precip_cmap, levels=levels, zorder=1,
                        vmin=-np.inf, vmax=np.inf, extend='both',
                        )
ax.set_title(f'{title}\n{calendar.month_name[month]} '
             f'{first_day}-{last_day}')
ax.set_extent(extent)
# cjs.graphics.add_shapes(ax=ax, shp_name='countyl010g', facecolor='none')
ax.outline_patch.set_visible(True)

# ibt = ibtracs.Ibtracs(in_path='last3years', download_data=True)
# ibt.map_one_storm(ax=ax, sid='2020294N12280', add_date_labels=True,
#                        zorder=99)
# ibt.map_one_storm(ax=ax, sid='2020303N12301', add_date_labels=True,
#                        zorder=99)
# cjs.raster.add_shaded_relief(ax=ax)

divider = make_axes_locatable(ax)
cax = divider.append_axes("bottom", size="5%", pad=0.10, axes_class=plt.Axes)
fig.add_axes(cax)
str_levels = [str(lev) for lev in levels]
cb = plt.colorbar(plot, cax=cax, orientation='horizontal',
                  panchor=False, label='Precipitation (inches)')
cb.set_ticks(levels)
cb.set_ticklabels(str_levels)

cjs.text.tstamp('Add labels')
plt.text(1.00, -0.30, 'Data Source: 5km Gridded\n'
                     '(nClimGrid-Daily) Beta',
         fontsize=7, ha='right', transform=ax.transAxes)
today_string = (f'{datetime.now().strftime("Created: %a %b %d %Y")}\n'
                f'{file_type.capitalize()} data')
plt.text(0.0, -0.30, today_string,
         fontsize=7, ha='left', transform=ax.transAxes)


cjs.text.tstamp('Drawing figure')
# plt.tight_layout()
plt.savefig(f'figures/final/{title.replace(" ", "_")}.png',
            dpi=200, bbox_inches='tight')
plt.show()

cjs.text.tstamp('Thank you, come again.')

