""" Draw a map of precipitation anomalies for a range of days."""

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
import os
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cjs.text
import cjs.graphics
import cjs.fileio
from datetime import datetime
from download_nclimgrid import download_nclimgrid_daterange
import argparse

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--first_date', default='2023-09-01')
parser.add_argument('--last_date', default='2023-09-30')
parser.add_argument('--dpi', default=200)
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--file_type', default='scaled')
args = parser.parse_args()
first_date = datetime.strptime(args.first_date, '%Y-%m-%d')
last_date = datetime.strptime(args.last_date, '%Y-%m-%d')
fig_dir = args.fig_dir
os.makedirs(fig_dir, exist_ok=True)
dpi = int(args.dpi)
file_type = args.file_type

clobber = False

cjs.text.tstamp('Here we go!')


in_paths = download_nclimgrid_daterange(first_date, last_date, clobber=clobber,
                                        file_type=file_type)
if any('prelim' in in_path for in_path in in_paths):
    file_type = 'Preliminary'
else:
    file_type = 'Scaled'

cjs.text.tstamp('Read and average totals')
total_ds = xr.open_mfdataset(in_paths)
dayofyear = total_ds.time.sel(time=slice(first_date, last_date)).dt.dayofyear
dayofyear_ind = dayofyear.values - 1
total_data = total_ds['prcp'].sel(time=slice(first_date, last_date)
                                  ).sum(dim='time').load()

cjs.text.tstamp('Read and average climatology')
clim_path = (f'{os.environ["DATA_DIR"]}/nclimgrid/v1-0-0/normals/'
             f'prcp-1991_2020-daily-normals-v1.0.nc')
clim_ds = xr.open_dataset(clim_path)
clim_data = clim_ds['dlyprcp_norm'].isel(time=dayofyear_ind).sum(dim='time')

cjs.text.tstamp('Anomalize')
anom_data = 100. * total_data / clim_data
# Take care of infinities when the observed prcp is nonzero
anom_data = anom_data.where((total_data == 0) | (clim_data != 0), 600)
# and when observed is zero
anom_data = anom_data.where((total_data != 0) | (clim_data != 0), 0)
anom_data = anom_data.where(total_ds['prcp'][0].notnull())


cjs.text.tstamp('Draw map')
ax = plt.axes(projection=cjs.graphics.conus_projection)
cjs.graphics.add_conus_map(ax=ax)

cjs.text.tstamp('Draw data')
levels = [5, 10, 25, 50, 75, 100, 125, 150, 200, 300, 500]
plot = anom_data.plot.imshow(levels=levels, ax=ax, transform=ccrs.PlateCarree(),
                      cmap='BrBG', zorder=2, add_labels=False,
                      extend='both',
                      cbar_kwargs={'orientation': 'horizontal',
                                   'label': 'Percent',
                                   'pad': 0.0})
plot.colorbar.set_ticklabels(levels)
plot.colorbar.set_ticks(levels)

cjs.text.tstamp('Add labels')
if (first_date.month == last_date.month):
    date_string = f'{first_date.strftime("%b %-d")}-' \
                  f'{last_date.strftime("%-d %Y")}'
else:
    date_string = f'{first_date.strftime("%b %-d")}-' \
                  f'{last_date.strftime("%b %-d %Y")}'
plt.text(0.5, 1.11, 'Precipitation Percent of Average',
         fontsize=14, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.05, date_string,
         fontsize=11, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.00, 'Average Period: 1991-2020',
         fontsize=9, ha='center', transform=ax.transAxes)
plt.text(1.1, -0.25, 'Data Source: 5km Gridded\n'
                     'nClimGrid-Daily v1-0-0',
         fontsize=7, ha='right', transform=ax.transAxes)
today_string = (f'{datetime.now().strftime("Created: %a %b %d %Y")}\n'
                f'{file_type.capitalize()} data')
plt.text(-0.1, -0.25, today_string,
         fontsize=7, ha='left', transform=ax.transAxes)

cjs.text.tstamp('Save figure')
plt.tight_layout()

image_path = (f'{fig_dir}/prcp-pon-{first_date.strftime("%Y-%m%d")}'
              f'-{last_date.strftime("%m%d")}.png')
plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
# plt.show()
plt.close()

logo_path = f'{os.environ["DATA_DIR"]}/logos/ncei_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.17, logo_x=0.905, logo_y=0.56)

logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.09, logo_x=0.905, logo_y=0.73)

cjs.text.tstamp('Thank you, come again.')

