""" Draw a map of temperature anomalies for a range of days."""

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
from matplotlib import cm
import os
import matplotlib.pyplot as plt
import numpy as np
import cartopy.crs as ccrs
import cjs.text
from download_nclimgrid import download_nclimgrid_daterange
import cjs.graphics
from datetime import datetime
import argparse
import matplotlib.colors as mcolors

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--first_date', default='2022-05-01')
parser.add_argument('--last_date', default='2022-07-28')
parser.add_argument('--dpi', default=150)
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--file_type', default='scaled')
args = parser.parse_args()
first_date = datetime.strptime(args.first_date, '%Y-%m-%d')
last_date = datetime.strptime(args.last_date, '%Y-%m-%d')
fig_dir = args.fig_dir
os.makedirs(fig_dir, exist_ok=True)
dpi = int(args.dpi)
file_type = args.file_type

clobber = False

cjs.text.tstamp('Here we go!')


in_paths = download_nclimgrid_daterange(first_date, last_date, clobber=clobber,
                                        file_type=file_type)
if any('prelim' in in_path for in_path in in_paths):
    file_type = 'Preliminary'
else:
    file_type = 'Scaled'

cjs.text.tstamp('Read and average totals')
total_ds = xr.open_mfdataset(in_paths)
total_data = total_ds['prcp'].sel(time=slice(first_date, last_date)
                                  ).sum(dim='time').load()
total_data = total_data.where(total_ds['prcp'][0].load().notnull())


cjs.text.tstamp('Reading past years')
oldest_year = 1951
this_year = datetime.today().year
ref_data = total_data.expand_dims(
    dim={'year': np.arange(oldest_year, this_year)}, axis=0).copy()
ref_data[:] = np.nan
for ref_year in range(oldest_year, this_year):
    first_ref_date = datetime(ref_year, first_date.month, first_date.day)
    # Check if the original date is February 29th and the reference year is not a leap year.
    if last_date.month == 2 and last_date.day == 29 and ref_year % 4 != 0:
        # It's not a leap year, so use March 1st instead.
        last_ref_date = datetime(ref_year, 3, 1)
    else:
        # For any other date, or if the reference year is a leap year, use the same date.
        last_ref_date = datetime(ref_year, last_date.month, last_date.day)

    in_paths = download_nclimgrid_daterange(first_ref_date, last_ref_date,
                                            clobber=clobber)
    ref_ds = xr.open_mfdataset(in_paths)
    ref_data.loc[ref_year] = ref_ds['prcp'].sel(
        time=slice(first_ref_date, last_ref_date)).sum(dim='time')


cjs.text.tstamp('Find percentiles')
pct_rank = (total_data > ref_data).sum(dim='year')
pct_rank = pct_rank / ref_data.year.size
pct_rank = pct_rank.where(total_data.notnull())
pct_rank = xr.where((ref_data.min(dim='year') == 0.00) & (total_data == 0.00),
                    -1, pct_rank)

cjs.text.tstamp('Draw map')
ax = plt.axes(projection=cjs.graphics.conus_projection)
cjs.graphics.add_conus_map(ax=ax, facecolor='none')

cjs.text.tstamp('Draw data')
levels = [-2, -0.5, 1e-10, .1, .33, .67, .9, 1-1e-10, 2]
brbg = cm.get_cmap('BrBG', 256)(np.linspace(0.1, 0.9, 9))
newcolors = np.append([mcolors.to_rgba('lightgray')], brbg, axis=0)
newcmp = mcolors.ListedColormap(newcolors)
plot = pct_rank.plot.imshow(ax=ax, transform=ccrs.PlateCarree(),
                            cmap=newcmp, levels=levels, zorder=2,
                            extend='neither', vmin=-np.inf, vmax=np.inf,
                            cbar_kwargs={'orientation': 'horizontal',
                                         'label': '',
                                         'pad': 0.0}
                            )
ax.set_title('')
labels = ['Record\nDriest\n(Tie 0.00")',
          'Record\nDriest', 'Much\nBelow\nAverage', 'Below\nAverage',
          'Near\nAverage', 'Above\nAverage', 'Much\nAbove\nAverage',
          'Record\nWettest',
          ]
ticks = [(l0 + l1) / 2 for l0, l1 in zip(levels[:-1], levels[1:])]
plot.colorbar.set_ticks(ticks)
plot.colorbar.set_ticklabels(labels)
plot.colorbar.ax.tick_params(labelsize=8)
print(f'{pct_rank.min()} -- {pct_rank.max()}')

# cmap = plt.cm.rdbu_r
# legend_elements = [
#     patch(facecolor=cmap(0.0), edgecolor='k', label='record\ncoldest'),
#     patch(facecolor=cmap(1./6), edgecolor='k', label='much\nbelow\naverage'),
#     patch(facecolor=cmap(2./6), edgecolor='k', label='below\naverage'),
#     patch(facecolor=cmap(3./6), edgecolor='k', label='near\naverage'),
#     patch(facecolor=cmap(4./6), edgecolor='k', label='above\naverage'),
#     patch(facecolor=cmap(5./6), edgecolor='k', label='much\nabove\naverage'),
#     patch(facecolor=cmap(1.0), edgecolor='k', label='record\warmest'),
# ]
# ax.legend(handles=legend_elements, loc='center', bbox_to_anchor=(0.5, -0.10),
#           ncol=7)
#


# ibt = ibtracs.Ibtracs(in_path='last3years', download_data=True)
# ibt.map_one_storm(ax=ax, sid='2021236N12296', add_date_labels=False,
#                   zorder=99, add_legend=False, markercolor='none')
# ibt.map_one_storm(ax=ax, sid='2021220N11311', add_date_labels=False,
#                   zorder=99, add_legend=False, markercolor='none')

cjs.text.tstamp('Add labels')
if (first_date.month == last_date.month):
    date_string = f'{first_date.strftime("%b %-d")}-' \
                  f'{last_date.strftime("%-d %Y")}'
else:
    date_string = f'{first_date.strftime("%b %-d")}-' \
                  f'{last_date.strftime("%b %-d %Y")}'
plt.text(0.5, 1.11, 'Total Precipitation Percentiles',
         fontsize=14, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.05, date_string,
         fontsize=11, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.00, f'Ranking Period: {oldest_year}-{this_year}',
         fontsize=9, ha='center', transform=ax.transAxes)
plt.text(1.1, -0.35, 'Data Source: 5km Gridded\n'
                     'nClimGrid-Daily v1-0-0',
         fontsize=7, ha='right', transform=ax.transAxes)
today_string = (f'{datetime.now().strftime("Created: %a %b %d %Y")}\n'
                f'{file_type} data')
plt.text(-0.1, -0.35, today_string,
         fontsize=7, ha='left', transform=ax.transAxes)

cjs.text.tstamp('Save figure')
plt.tight_layout()

image_path = (f'{fig_dir}/prcp-rank-{first_date.strftime("%Y-%m%d")}'
              f'-{last_date.strftime("%m%d")}.png')
plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
# plt.show()
plt.close()

logo_path = f'{os.environ["DATA_DIR"]}/logos/ncei_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.15, logo_x=0.905, logo_y=0.51)

logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.09, logo_x=0.905, logo_y=0.68)

cjs.text.tstamp('Thank you, come again.')

