import matplotlib.pyplot as plt
from rmm_phase_background import rmm_phase_background
from datetime import datetime
from rmm_draw import rmm_draw
import pandas as pd
import requests
from io import StringIO
import argparse
import os

def main(first_date, last_date, fig_dir, dpi, use_bom_not_local=True):
    if use_bom_not_local:
        print('Reading BOM RMM')
        df = read_bom()
    else:
        print('Reading local RMM')
        df = read_local_rmm()

    print(df)
    df["date"] = pd.to_datetime(df[["year", "month", "day"]])
    df.set_index("date", inplace=True)
    rmm_pc = [df["RMM1"], df["RMM2"]]

    # Call the function
    fig, ax = plt.subplots()
    print('Drawing background')
    rmm_phase_background(ax)
    print('Drawing Data')
    rmm_draw(ax, rmm_pc, first_date, last_date)
    print('Saving')
    os.makedirs(fig_dir, exist_ok=True)
    image_path = (f'{fig_dir}/mjo-{first_date.strftime("%Y-%m%d")}'
                  f'-{last_date.strftime("%m%d")}.png')
    plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
    print('Thank you. Come again.')


def read_bom(url="http://www.bom.gov.au/clim_data/IDCKGEM000/rmm.74toRealtime.txt"):
    # Mimic a browser in the User-Agent to bypass the server restriction
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"}

    response = requests.get(url, headers=headers)

    # Make sure we got a valid response
    if(response.status_code == 200):
        data = StringIO(response.text)

        # Define the column names
        col_names = ["year", "month", "day", "RMM1", "RMM2", "phase", "amplitude", "note"]

        # Define the width of each field
        widths = [12, 12, 12, 16, 16, 12, 12, 50]

        # Read the data
        df = pd.read_fwf(data, widths=widths, skiprows=2, names=col_names)
        return df
    else:
        print("Failed to download data. HTTP response code: ", response.status_code)


def read_local_rmm(in_path='/glfs0/anon-ftp/pub/carl/monitor/mjo/wh04.orig.csv'):
    col_names = ["year", "month", "day", "RMM1", "RMM2", "phase", "amplitude",
                 "note"]
    df = pd.read_csv(in_path, names=col_names)
    return df


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--first_date', default='2024-08-01')
    parser.add_argument('--last_date', default='2024-11-19')
    parser.add_argument('--fig_dir', default='synoptic/')
    parser.add_argument('--dpi', default=300)

    args = parser.parse_args()
    first_date = datetime.strptime(args.first_date, '%Y-%m-%d')
    last_date = datetime.strptime(args.last_date, '%Y-%m-%d')
    fig_dir = args.fig_dir
    dpi = int(args.dpi)

    main(first_date=first_date, last_date=last_date, fig_dir=fig_dir, dpi=dpi)
