""" Draw a map of the difference between scaled and prelim tavg. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
import os
import matplotlib.pyplot as plt
import numpy as np
import cartopy.crs as ccrs
import calendar
import cjs.text
import cjs.graphics
import cjs.fileio
from datetime import datetime
import argparse

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=(datetime.today().year))
parser.add_argument('--month', default=(datetime.today().month - 1))
parser.add_argument('--first_day', default=22)
parser.add_argument('--last_day', default=23)
parser.add_argument('--dpi', default=200)
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--no_terrain', dest='terrain', action='store_false')
parser.set_defaults(terrain=False)
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
if month < 1:
    month += 12
    year -= 1
first_day = int(args.first_day)
last_day = int(args.last_day)
fig_dir = args.fig_dir
dpi = int(args.dpi)
add_terrain = args.terrain

clobber = False

time_range = slice(f'{year}-{month:02}-{first_day:02}',
                   f'{year}-{month:02}-{last_day:02}')


cjs.text.tstamp('Here we go!')


cjs.text.tstamp('Downloading')
file_type = 'scaled'
file_name = f'tavg-{year}{month:02}-grd-{file_type}.nc'
scaled_path = f'{os.environ["DATA_DIR"]}/nclimgrid/beta/tavg/{file_name}'
ftp_path = (f'ftp://ftp.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
            f'{year}/{month:02}/{file_name}')
web_path = (f'https://www1.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
            f'{year}/{month:02}/{file_name}')
download_err = cjs.fileio.download_file(ftp_path, scaled_path, clobber=clobber)
file_type = 'prelim'
file_name = f'tavg-{year}{month:02}-grd-{file_type}.nc'
prelim_path = f'{os.environ["DATA_DIR"]}/nclimgrid/beta/tavg/{file_name}'
web_path = (f'https://www1.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
            f'{year}/{month:02}/{file_name}')
download_err = cjs.fileio.download_file(web_path, prelim_path, clobber=clobber)

cjs.text.tstamp('Read and average scaled')
scaled_ds = xr.open_dataset(scaled_path)
dayofyear = scaled_ds.time.sel(time=time_range).dt.dayofyear
scaled_data = scaled_ds['tavg'].sel(time=time_range).mean(dim='time')

cjs.text.tstamp('Read and average prelim')
prelim_ds = xr.open_dataset(prelim_path)
prelim_data = prelim_ds['tavg'].sel(time=time_range).mean(dim='time')

cjs.text.tstamp('Anomalize')
diff_data = (scaled_data - prelim_data) * 9 / 5

cjs.text.tstamp('Draw map')
ax = plt.axes(projection=cjs.graphics.conus_projection)
cjs.graphics.add_conus_map(ax=ax, facecolor='none')
if add_terrain:
    cjs.raster.add_shaded_relief(ax=ax)

cjs.text.tstamp('Draw data')
levels = list(np.arange(-5,6)*2)
plot = diff_data.plot.imshow(ax=ax, transform=ccrs.PlateCarree(),
                      cmap='RdBu_r', levels=levels, zorder=2, extend='both',
                      vmin=-np.inf, vmax=np.inf,
                      cbar_kwargs={'orientation': 'horizontal',
                                   'label': 'Degrees Fahrenheit',
                                   'pad': 0.0}
                      )
plot.colorbar.set_ticklabels(levels)
plot.colorbar.set_ticks(levels)
print(f'{diff_data.min()} -- {diff_data.max()}')

cjs.text.tstamp('Add labels')
date_string = f'{calendar.month_name[month]} {first_day}-{last_day} {year}'
plt.text(0.5, 1.11, 'Scaled minus Prelim Average Temperature',
         fontsize=14, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.05, date_string,
         fontsize=11, ha='center', transform=ax.transAxes)
# plt.text(0.5, 1.00, 'Average Period: 1981-2010',
#          fontsize=9, ha='center', transform=ax.transAxes)
plt.text(1.1, -0.25, 'Data Source: 5km Gridded\n'
                     '(nClimGrid-Daily) Beta',
         fontsize=7, ha='right', transform=ax.transAxes)
today_string = (f'{datetime.now().strftime("Created: %a %b %d %Y")}\n'
                f'{file_type.capitalize()} data')
plt.text(-0.1, -0.25, today_string,
         fontsize=7, ha='left', transform=ax.transAxes)

cjs.text.tstamp('Save figure')
plt.tight_layout()

image_path = (f'{fig_dir}/tave-scaling-{year}-{month:02}{first_day:02}'
              f'-{month:02}{last_day:02}.png')
plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
# plt.show()
plt.close()

cjs.text.tstamp('Thank you, come again.')

