""" Draw a map of temperature total and anomalies for a range of days."""

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
import os
import matplotlib.pyplot as plt
import numpy as np
import cartopy.crs as ccrs
import calendar
import cjs.text
import cjs.fileio
import cjs.graphics
from datetime import datetime
import argparse

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', default=(datetime.today().year))
parser.add_argument('--month', default=(datetime.today().month - 1))
parser.add_argument('--first_day', default=16)
parser.add_argument('--last_day', default=16)
parser.add_argument('--dpi', default=200)
parser.add_argument('--var_name', default='tmax')
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--no_terrain', dest='terrain', action='store_false')
parser.set_defaults(terrain=False)
args = parser.parse_args()
year = int(args.year)
month = int(args.month)
if month < 1:
    month += 12
    year -= 1
first_day = int(args.first_day)
last_day = int(args.last_day)
fig_dir = args.fig_dir
var_name = args.var_name
dpi = int(args.dpi)
add_terrain = args.terrain

clobber = False

time_range = slice(f'{year}-{month:02}-{first_day:02}',
                   f'{year}-{month:02}-{last_day:02}')


cjs.text.tstamp('Here we go!')


cjs.text.tstamp('Downloading')
file_type = 'scaled'
file_name = f'{var_name}-{year}{month:02}-grd-{file_type}.nc'
in_path = f'{os.environ["DATA_DIR"]}/nclimgrid/beta/{var_name}/{file_name}'
ftp_path = (f'ftp://ftp.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
            f'{year}/{month:02}/{file_name}')
web_path = (f'https://www1.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
            f'{year}/{month:02}/{file_name}')
download_err = cjs.fileio.download_file(ftp_path, in_path, clobber=clobber)
if download_err:
    file_type = 'prelim'
    file_name = f'{var_name}-{year}{month:02}-grd-{file_type}.nc'
    in_path = f'{os.environ["DATA_DIR"]}/nclimgrid/beta/{var_name}/{file_name}'
    ftp_path = (f'https://www1.ncdc.noaa.gov/pub/data/daily-grids/beta/by-month/'
                f'{year}/{month:02}/{file_name}')
    download_err = cjs.fileio.download_file(ftp_path, in_path, clobber=clobber)

cjs.text.tstamp('Read and average totals')
total_ds = xr.open_dataset(in_path)
dayofyear = total_ds.time.sel(time=time_range).dt.dayofyear
total_data = total_ds[var_name].sel(time=time_range).mean(dim='time')

total_data = (total_data * 9 /5) + 32

cjs.text.tstamp('Draw map')
ax = plt.axes(projection=cjs.graphics.conus_projection)

cjs.graphics.add_conus_map(ax=ax, facecolor='none')
if add_terrain:
    cjs.raster.add_shaded_relief(ax=ax)

cjs.text.tstamp('Draw data')
levels = list(np.arange(-30,35,5))
plot = total_data.plot.imshow(ax=ax, transform=ccrs.PlateCarree(),
                             cmap='GnBu_r', levels=levels, zorder=2,
                             extend='both', vmin=-np.inf, vmax=np.inf,
                             cbar_kwargs={'orientation': 'horizontal',
                                          'label': 'Degrees Fahrenheit',
                                          'pad': 0.0}
                             )
print(f'{total_data.min()} -- {total_data.max()}')

cjs.text.tstamp('Add labels')
if first_day == last_day:
    image_path = (f'{fig_dir}/{var_name}-total-{year}-{month:02}{first_day:02}'
                  f'.{file_type}.png')
    date_string = f'{calendar.month_name[month]} {first_day} {year}'
else:
    image_path = (f'{fig_dir}/{var_name}-total-{year}-{month:02}{first_day:02}'
                  f'-{month:02}{last_day:02}.{file_type}.png')
    date_string = f'{calendar.month_name[month]} {first_day}-{last_day} {year}'
ax.set_title(date_string)

cjs.text.tstamp('Save figure')
plt.tight_layout()

plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
# plt.show()
plt.close()

cjs.text.tstamp('Thank you, come again.')

