""" Download several climate indices for this month and plot them. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

# Teleconnections from:
# https://www.cpc.ncep.noaa.gov/products/precip/CWlink/daily_ao_index/teleconnections.shtml

# Nino 3.4 from:
# https://www.cpc.ncep.noaa.gov/data/indices/
# https://www.cpc.ncep.noaa.gov/products/analysis_monitoring/ensostuff/detrend.nino34.ascii.txt


import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
from matplotlib import dates as mdates
import matplotlib.ticker as plticker
import os
from datetime import datetime
import cjs.text
import cjs.graphics
import cjs.fileio


cjs.text.tstamp('Here we go!')
# These are some parameters that could be useful to have up top
year = datetime.today().year - 1
months = np.arange(1, 13)
time_range = slice(f'{year}-01-01', f'{year}-12-31')

fig_dir = f'/glfs0/anon-ftp/pub/carl/monitor/synoptic-{year}13'
# fig_dir = f'synoptic/{year}/'
os.makedirs(fig_dir, exist_ok=True)
image_path = f'{fig_dir}/teleconnections-{year}.png'

telecon_names = np.array(['AO', 'NAO', 'PNA', 'WPO', 'EPO', 'Nino3.4'])
telecon_is_sst = np.array([telecon_name == 'Nino3.4' for telecon_name in
                           telecon_names])

file_names = {
    'AO': 'monthly.ao.index.b50.current.ascii',
    'NAO': 'norm.nao.monthly.b5001.current.ascii',
    'PNA': 'norm.pna.monthly.b5001.current.ascii',
    'Nino3.4': 'detrend.nino34.ascii',
}


daily_indices = ['WPO', 'EPO']

units = {
    'AO': 'AO Index',
    'NAO': 'NAO Index',
    'PNA': 'PNA Index',
    'WPO': 'WPO Index',
    'EPO': 'EPO Index',
    'Nino3.4': 'Nino 3.4 (°C)',
}

signif_levels = {
    'AO': 1,
    'NAO': 1,
    'PNA': 1,
    'EPO': 1,
    'WPO': 1,
    'Nino3.4': 0.5,
}

remote_paths = {
    'AO': 'https://www.cpc.ncep.noaa.gov/products/precip/CWlink'
          '/daily_ao_index/monthly.ao.index.b50.current.ascii',
    'NAO': 'https://www.cpc.ncep.noaa.gov/products/precip/CWlink/pna/'
           'norm.nao.monthly.b5001.current.ascii',
    'PNA': 'https://www.cpc.ncep.noaa.gov/products/precip/CWlink/pna/'
           'norm.pna.monthly.b5001.current.ascii',
    'Nino3.4': 'https://www.cpc.ncep.noaa.gov/products/analysis_monitoring'
               '/ensostuff/detrend.nino34.ascii.txt',
}

local_path = f'{os.environ["DATA_DIR"]}/clim_indices/monthly/'
os.makedirs(local_path, exist_ok=True)

fig, axs = plt.subplots(len(telecon_names), figsize=[4.0, 12.0])
clobber = True
for (ax, telecon_name) in zip(axs, telecon_names):


    if telecon_name in daily_indices:
        remote_path = f'ftp://ftp.cdc.noaa.gov/Public/map/teleconnections/' \
                      f'{telecon_name.lower()}.reanalysis.t10trunc.1948-present.txt'
        standardize = 'monthly'
        sep = '\s+'
        header = None
        local_path = f'{os.environ["DATA_DIR"]}/clim_indices/cpc_daily/'
        clim_range = slice('1991-01-01', '2020-12-31')
        daily_df = cjs.fileio.download_year_month_day_data(
            remote_path, local_path, clobber=True,
            standardize=standardize, clim_range=clim_range,
            sep=sep, header=header, quiet=True).to_dataframe(name=telecon_name)

        df = (daily_df
              .resample('MS')[telecon_name]
              .mean()
              .reset_index()
              .set_index('time')
              )


    else:
        cjs.fileio.download_file(
            remote_paths[telecon_name],
            local_path=f'{local_path}/{file_names[telecon_name]}',
            clobber=clobber)
        header = 0 if (telecon_name == 'Nino3.4') else None
        df = pd.read_csv(f'{local_path}{file_names[telecon_name]}',
                            delim_whitespace=True, header=header)
        df = df.rename(columns={
            df.columns[0]: 'year',
            df.columns[1]: 'month',
            df.columns[-1]: telecon_name,
        })
        df.index = pd.to_datetime(df['year'].astype('str')
                                  + df['month'].astype('str'),
                                  format='%Y%m')
    print(df)
    sub_df = df[time_range]

    ax.plot_date(sub_df.index, sub_df[telecon_name], color='none')
    ax.fill_between(sub_df.index, sub_df[telecon_name], 0, where=sub_df[telecon_name] > 0,
                    facecolor='tab:red', interpolate=True)
    ax.fill_between(sub_df.index, sub_df[telecon_name], 0, where=sub_df[telecon_name] < 0,
                    facecolor='tab:blue', interpolate=True)
    ax.set_xlim(sub_df.index[0], sub_df.index[-1])
    ax.plot(ax.get_xlim(), [0, 0], color='k', linewidth=1)
    ax.plot(ax.get_xlim(),
            [signif_levels[telecon_name], signif_levels[telecon_name]],
            '--', color='k', linewidth=0.5)
    ax.plot(ax.get_xlim(),
            [-signif_levels[telecon_name], -signif_levels[telecon_name]],
            '--', color='k', linewidth=0.5)
    ax.set_xlim(sub_df.index[0], sub_df.index[-1])
    ax.xaxis.set_major_formatter(mdates.DateFormatter('%b'))
    ax.set_ylabel(units[telecon_name])
    ax.set_title(telecon_name, loc='center')
    loc = plticker.MultipleLocator(base=signif_levels[telecon_name])
    ax.yaxis.set_major_locator(loc)

    for label in ax.get_xticklabels():
        label.set_rotation(30)
        label.set_horizontalalignment('right')


fig.text(0.50, 0.023, 'National Centers for Environmental Information',
         fontsize=6, ha='center')

plt.tight_layout(rect=(0.00, 0.04, 1, 1), h_pad=2)
# plt.savefig(image_path, dpi=200, bbox_inches='tight')
plt.savefig(image_path, dpi=200)
logo_path = f'{os.environ["DATA_DIR"]}/logos/noaa_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.04, logo_x=0.12, logo_y=0.97)

logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.04, logo_x=0.88, logo_y=0.97)

cjs.text.tstamp('Thank you, come again.')

