""" Download several climate indices for this month and plot them. """

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import numpy as np
from matplotlib import pyplot as plt
from matplotlib import dates as mdates
import matplotlib.ticker as plticker
import os
import calendar
from datetime import datetime
import argparse
import cjs.text
import cjs.fileio
import cjs.graphics
import matplotlib

matplotlib.rcParams.update({'font.size': 8})


def print_percentile(data_value, all_data, group_string):
    if data_value > 0:
        rank = 1 + (data_value < all_data).sum().item()
    else:
        rank = 1 + (data_value > all_data).sum().item()
    percentile = (1 - rank / float(len(all_data))) * 100
    print(f'{group_string:15}: {data_value:10.2f}    {rank:10}     '
          f' {percentile:10.1f}')


def download_and_read_telecon(telecon_name):
    if telecon_name in ['pna', 'nao', 'ao']:  # These are from CPC
        if telecon_name == 'ao':
            level = 'z1000'
        else:
            level = 'z500'
        remote_path = f'http://ftp.cpc.ncep.noaa.gov/cwlinks/' \
                      f'norm.daily.{telecon_name}.cdas.' \
                      f'{level}.19500101_current.csv'
        standardize = None
        sep = ','
        header = 0
    elif telecon_name in ['wpo', 'epo']:  # These are from ESRL
        remote_path = f'ftp://ftp.cdc.noaa.gov/Public/map/teleconnections/' \
                      f'{telecon_name}.reanalysis.t10trunc.1948-present.txt'
        standardize = 'monthly'
        sep = '\s+'
        header = None
    else:
        print(f'WARNING: Invalid telecon_name: {telecon_name}')
        return None

    local_path = f'{os.environ["DATA_DIR"]}/clim_indices/cpc_daily/'
    clim_range = slice('1991-01-01', '2020-12-31')
    data = cjs.fileio.download_year_month_day_data(
        remote_path, local_path, clobber=True,
        standardize=standardize, clim_range=clim_range,
        sep=sep, header=header, quiet=True)
    return data



cjs.text.tstamp('Here we go!')
# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--year', type=int, default=(datetime.today().year))
parser.add_argument('--month', type=int, default=(datetime.today().month))
parser.add_argument('--dpi', type=int, default=200)
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--first_days', type=int, nargs='+', default=[1, 8, 16, 23])
args = parser.parse_args()
year = args.year
month = args.month-1
print(args.first_days)
first_days = np.array(args.first_days)
fig_dir = args.fig_dir
os.makedirs(fig_dir, exist_ok=True)
# month = 3
if month < 1:
    month += 12
    year -= 1
dpi = int(args.dpi)

os.makedirs(fig_dir, exist_ok=True)

days_in_month = calendar.monthrange(year, month)[1]
time_range = slice(f'{year}-{month}-01', f'{year}-{month}-{days_in_month}')
first_days = np.append(first_days, days_in_month)

telecon_names = ['pna', 'ao', 'nao', 'wpo', 'epo']
# telecon_names = ['pna', 'ao', 'nao', 'epo']
# telecon_names = ['pna', 'ao', 'nao', ]
# telecon_names = ['wpo', 'epo']

fig, axs = plt.subplots(len(telecon_names), figsize=[4.0, 8.0])

for (ax, telecon_name) in zip(axs, telecon_names):
    data = download_and_read_telecon(telecon_name)

    monthly_data = data.resample(time='m').mean()
    monthly_std_data = ((monthly_data - monthly_data.mean(dim='time'))
                        / monthly_data.std(dim='time'))

    sub_data = data.sel(time=time_range)
    ax.plot_date(sub_data['time'], sub_data, color='none')
    ax.fill_between(sub_data['time'], sub_data, 0, where=sub_data > 0,
                    facecolor='tab:red', interpolate=True)
    ax.fill_between(sub_data['time'], sub_data, 0, where=sub_data < 0,
                    facecolor='tab:blue', interpolate=True)
    ax.set_xlim(sub_data.time[0], sub_data.time[-1])
    ax.plot(ax.get_xlim(), [0, 0], color='k', linewidth=1)
    ax.plot(ax.get_xlim(), [-1, -1], '--', color='k', linewidth=0.5)
    ax.plot(ax.get_xlim(), [1, 1], '--', color='k', linewidth=0.5)
    ax.set_ylim(ax.get_ylim())
    ax.set_xlim(sub_data.time[0], sub_data.time[-1])
    ax.xaxis.set_minor_locator(mdates.DayLocator(interval=1))
    ax.xaxis.set_major_locator(mdates.DayLocator(bymonthday=first_days))
    # ax.xaxis.set_major_formatter(mdates.DateFormatter('%-d'))
    ax.xaxis.set_major_formatter(mdates.DateFormatter('%b-%-d'))
    # if telecon_name != telecon_names[-1]:
    #     ax.set_xticklabels([])
    # else:
    #     ax.tick_params(axis='x', labelrotation=270)
    # ax.tick_params(axis='x', labelrotation=45, rotation_mode='anchor')
    # ax.set_xticklabels(ax.get_xticklabels(), rotation=45, ha='right',
    #                    rotation_mode='anchor')
    for label in ax.get_xticklabels():
        label.set_rotation(30)
        label.set_horizontalalignment('right')
    ax.set_title(telecon_name.upper(), loc='left')
    ax.set_title(f'Monthly mean: '
                 f'{monthly_std_data.sel(time=time_range.stop).values:+.1f}',
                 loc='right')
    ax.set_ylabel(f'{telecon_name.upper()} index')
    # ax.set_xlabel(calendar.month_name[month])

    print()
    print(telecon_name.upper())
    curr_month_value = monthly_std_data.sel(time=time_range.stop).values
    print_percentile(curr_month_value, monthly_std_data, 'Month - ALL')
    is_in_month = monthly_std_data['time.month'] == month
    print_percentile(curr_month_value, monthly_std_data[is_in_month],
                     f'Month - {calendar.month_abbr[month]}')

    daily_max = sub_data.max().item()
    print_percentile(daily_max, data, 'Max - ALL')
    is_in_month = data['time.month'] == month
    print_percentile(daily_max, data[is_in_month],
                     f'Max - {calendar.month_abbr[month]}')

    daily_min = sub_data.min().item()
    print_percentile(daily_min, data, 'Min - ALL')
    is_in_month = data['time.month'] == month
    print_percentile(daily_min, data[is_in_month],
                     f'Min - {calendar.month_abbr[month]}')
    print()

ylim = [-2, 2]
for ax in axs:
    ylim[0] = min(ylim[0], ax.get_ylim()[0])
    ylim[1] = max(ylim[1], ax.get_ylim()[1])
if (ylim[1] - ylim[0] > 6):
    loc = plticker.MultipleLocator(base=2.0)
else:
    loc = plticker.MultipleLocator(base=1.0)
for ax in axs:
    ax.set_ylim(ylim)
    ax.yaxis.set_major_locator(loc)
    for first_day in first_days:
        first_day_dt = datetime(year, month, first_day)
        ax.plot([first_day_dt, first_day_dt], ax.get_ylim(),
                '--', color='k', linewidth=0.5)

# fig.suptitle(f'Standardized teleconnection amplitudes\n'
#           f' {calendar.month_name[month]} {year}')
# plt.subplots_adjust(bottom=0.50)
# fig.text(0.02, 0.5, 'Standard Deviations', va='center', rotation='vertical',
#          size='large')
fig.text(0.50, 0.023, 'National Centers for Environmental Information',
         fontsize=6, ha='center')

plt.tight_layout(rect=(0.00, 0.04, 1, 1), h_pad=2)

# plt.savefig(f'synoptic/final/teleconnections-{year}-{month:02}.png', dpi=200,)
image_path = f'{fig_dir}/teleconnections-{year}-{month:02}.png'
# plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
plt.savefig(image_path, dpi=dpi)
# plt.show()
plt.close()

logo_path = f'{os.environ["DATA_DIR"]}/logos/noaa_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.04, logo_x=0.12, logo_y=0.97)

logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                      logo_size=0.04, logo_x=0.88, logo_y=0.97)

cjs.text.tstamp('Thank you, come again.')
