""" Draw a map of temperature anomalies for a range of days."""

__author__ = "Carl Schreck"
__email__ = "cjschrec@ncsu.edu"
__copyright__ = "Copyright 2019, North Carolina State University"
__license__ = "BSD-3.0"

import xarray as xr
import os
import matplotlib.pyplot as plt
import matplotlib.path as mpath
import numpy as np
import cartopy.crs as ccrs
import matplotlib as mpl
from datetime import datetime
from dateutil import rrule
import argparse
import calendar
import cjs.text
import cjs.fileio
import cjs.graphics
import cjs.array_tools

# These are some parameters that could be useful to have up top
parser = argparse.ArgumentParser()
parser.add_argument('--first_date', default='2023-06-01')
parser.add_argument('--last_date', default='2023-08-31')
parser.add_argument('--dpi', default=400)
parser.add_argument('--fig_dir', default='synoptic/')
parser.add_argument('--image_name', default='None')
parser.add_argument('--region', default='namer')
args = parser.parse_args()
first_date = datetime.strptime(args.first_date, '%Y-%m-%d')
last_date = datetime.strptime(args.last_date, '%Y-%m-%d')
fig_dir = args.fig_dir
os.makedirs(fig_dir, exist_ok=True)
image_name = args.image_name
region = args.region
dpi = int(args.dpi)

days_in_month = calendar.monthrange(first_date.year, first_date.month)[1]

os.makedirs(fig_dir, exist_ok=True)

clobber = False


cjs.text.tstamp('Here we go!')


in_paths = []
for dt in rrule.rrule(rrule.YEARLY, dtstart=first_date, until=last_date):
    cjs.text.tstamp('Downloading')
    file_name = f'hgt.{dt.year}.nc'
    in_path = f'{os.environ["DATA_DIR"]}/ncep1/esrl/daily/{file_name}'
    ftp_path = (f'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis.dailyavgs/'
                f'pressure/{file_name}')
    download_err = cjs.fileio.download_file(ftp_path, in_path, clobber=clobber)
    in_paths.append(in_path)

cjs.text.tstamp('Read and average totals')
print(in_paths)
total_ds = xr.open_mfdataset(in_paths)
dayofyear = total_ds.time.sel(time=slice(first_date, last_date)).dt.dayofyear
total_data = total_ds['hgt'].sel(time=slice(first_date, last_date), level=500
                                 ).mean(dim='time')
total_data = total_data.drop('level')

cjs.text.tstamp('Read and average climatology')
clim_path = f'{os.environ["DATA_DIR"]}/ncep1/clim/hgt500.clim.1991.2020.nc'
clim_ds = xr.open_dataset(clim_path)
clim_data = clim_ds['hgt500'].sel(year_day=dayofyear.values).mean(
    dim='year_day')

cjs.text.tstamp('Anomalize')
anom_data = total_data - clim_data

anom_data = cjs.array_tools.add_cyclic(anom_data)
total_data = cjs.array_tools.add_cyclic(total_data)


cjs.text.tstamp('Draw map')
plt.figure(figsize=[6.4,6.0])
if region == 'nhem':
    ax = plt.axes(projection=ccrs.NorthPolarStereo(central_longitude=-90.5))
    ax.set_extent((-180, 180, 15, 90), crs=ccrs.PlateCarree())
    theta = np.linspace(0, 2 * np.pi, 100)
    center, radius = [0.5, 0.5], 0.5
    verts = np.vstack([np.sin(theta), np.cos(theta)]).T
    circle = mpath.Path(verts * radius + center)
    ax.set_boundary(circle, transform=ax.transAxes)
    gl = ax.gridlines(linestyle='--')
    gl.xlocator = mpl.ticker.FixedLocator([-180, -90, 0, 90, 180])
    gl.ylocator = mpl.ticker.FixedLocator([0, 30, 60, 90])
    shrink = 0.7
elif region == 'namer':
    ax = plt.axes(projection=ccrs.LambertConformal(
        central_longitude=-100, standard_parallels=(10,30)))
    ax.set_extent([-150, -50, 5, 67])
    gl = ax.gridlines(linestyle='--')
    gl.xlocator = mpl.ticker.FixedLocator(np.arange(0, 360, 20))
    gl.ylocator = mpl.ticker.FixedLocator(np.arange(-90, 100, 20))
    shrink = 1.0
elif region == 'global':
    ax = plt.axes(projection=ccrs.PlateCarree())
    ax.set_extent([-180, 180, -90, 90])
    shrink = 1.0


cjs.text.tstamp('Draw data')
if len(dayofyear) < 11:
    levels = list(np.arange(-6, 6) * 40 + 20)
elif len(dayofyear) < 20:
    levels = list(np.arange(-6, 6) * 20 + 10)
else:
    levels = list(np.arange(-6, 6) * 20 + 10)
plot = anom_data.plot.contourf(ax=ax, transform=ccrs.PlateCarree(),
                               cmap='RdBu_r', levels=levels, extend='both',
                               vmin=-np.inf, vmax=np.inf,
                               cbar_kwargs={'orientation': 'horizontal',
                                            'label': 'Meters',
                                            'pad': 0.02,
                                            'shrink': shrink
                                            })
plot.colorbar.set_ticklabels(levels)
plot.colorbar.set_ticks(levels)
clevels = np.arange(4000, 6000, 100)
if region == 'global':
    linewidths = 0.5
else:
    linewidths = 1.0
cont_plot = total_data.plot.contour(ax=ax, transform=ccrs.PlateCarree(),
                                    linewidths=linewidths, zorder=99,
                                    levels=clevels, colors=['black'])


cjs.graphics.add_map(ax, facecolor='none')

cjs.text.tstamp('Add labels')
if first_date.month == last_date.month:
    if (first_date.day == 1) and (last_date.day == days_in_month):
        date_string = f'{first_date.strftime("%B %Y")}'
    else:
        date_string = (f'{first_date.strftime("%b %-d")}-'
                      f'{last_date.strftime("%-d %Y")}')
else:
    date_string = f'{first_date.strftime("%b %-d")}-' \
                  f'{last_date.strftime("%b %-d %Y")}'
plt.text(0.5, 1.15, '500-hPa Heights and Anomalies',
         fontsize=12, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.09, date_string,
         fontsize=10, ha='center', transform=ax.transAxes)
plt.text(0.5, 1.04, 'Average Period: 1991-2020',
         fontsize=8, ha='center', transform=ax.transAxes)

# plt.tight_layout(rect=(0.00,0.05,1,1), h_pad=2)
today_string = f'{datetime.now().strftime("Created: %a %b %d %Y")}'
if region == 'global':
    text_y = -0.5
else:
    text_y = -0.3
plt.text(1.0, text_y, f'Data Source: NCEP-NCAR Reanalyis\n{today_string}',
         fontsize=7, ha='right', transform=ax.transAxes)
# plt.text(-0.0, -0.5, today_string,
#          fontsize=7, ha='left', transform=ax.transAxes)
plt.text(0.23, text_y, 'National Centers for\nEnvironmental\nInformation',
         fontsize=7, ha='left', transform=ax.transAxes)

cjs.text.tstamp('Save figure')

if image_name == 'None':
    image_path = (f'{fig_dir}/hgt500-{region}-{first_date.strftime("%Y-%m%d")}'
                  f'-{last_date.strftime("%m%d")}.png')
else:
    image_path = f'{fig_dir}/{image_name}'
plt.savefig(image_path, dpi=dpi, bbox_inches='tight')
cjs.graphics.image_resize(image_path=image_path, size=2000)

if region == 'nhem':
    logo_path = f'{os.environ["DATA_DIR"]}/logos/noaa_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.058, logo_x=0.05, logo_y=0.955)

    logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.058, logo_x=0.165, logo_y=0.955)
elif region == 'namer':
    logo_path = f'{os.environ["DATA_DIR"]}/logos/noaa_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.07, logo_x=0.05, logo_y=0.95)

    logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.07, logo_x=0.165, logo_y=0.95)
elif region == 'global':
    logo_path = f'{os.environ["DATA_DIR"]}/logos/noaa_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.09, logo_x=0.05, logo_y=0.94)

    logo_path = f'{os.environ["DATA_DIR"]}/logos/cisess_logo.png'
    cjs.graphics.add_logo(image_path=image_path, logo_path=logo_path,
                          logo_size=0.09, logo_x=0.165, logo_y=0.94)

thumb_path = image_path.replace('.png', '-t.png')
cjs.graphics.image_resize(image_path=image_path, out_path=thumb_path, size=350)

# plt.show()
plt.close()

cjs.text.tstamp('Thank you, come again.')

